import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.Serializable;
import java.beans.*;


/**Schnittstelle Serializable ist in Oberklasse Component bereits implementiert*/
public class DoubleParameterBean extends BasisBean
{
    private JLabel   aufkleber;
    private Textfeld textfeld;
    private Schieber schieber;

    /**
    Fr die einzustellenden Parameter mssen get- und set-Methoden bereitgestellt werden.
    */
    private double wert;
    private double minwert;
    private double maxwert;
    private String bezeichnung = null;

    public void aktualisieren()
    {
        textfeld.setText(""+wert);
        schieber.setWert(wert,minwert,maxwert);
    }

    public void setWert(double wert) 
    {     
        this.wert = wert;
        aktualisieren();
    }
    public double getWert() 
    {
        return this.wert;
    }

    public void setMinwert(double minwert) 
    {
        this.minwert=minwert;
        aktualisieren();
    }
    public double getMinwert() 
    {
        return this.minwert;
    }

    public void setMaxwert(double maxwert) 
    {
        this.maxwert=maxwert;
        aktualisieren();
    }
    public double getMaxwert() 
    {
        return this.maxwert;
    }


    public void setBezeichnung(String bezeichnung) 
    {
        aufkleber.setText(bezeichnung);
        firePropertyChange("bezeichnung",this.bezeichnung,bezeichnung);
        this.bezeichnung=bezeichnung;
    }
    public String getBezeichnung() 
    {
        return this.bezeichnung;
    }

    private void setWertIntern(String text)
    {
        double x = this.wert;
        double wert_alt = this.wert;
        boolean ok = true;
        try
        {
            x = Double.parseDouble(text);
        }
        catch(Exception e)
        {
            ok = false;
        }
        if( ok==true )
            this.wert = x;

        firePropertyChange("wert",wert_alt,this.wert);
        schieber.setWert(wert,minwert,maxwert);
    }

    public void setWertIntern(int iwert,int max_iwert)
    {
        double wert_alt = this.wert;
        this.wert = minwert + ((maxwert-minwert)*(double)iwert) / (double)max_iwert;

        firePropertyChange("wert",wert_alt,this.wert);
        textfeld.setText(""+wert);
    }

    /**
    JavaBeans mssen einen leeren Kontruktor besitzen.
    */
    public DoubleParameterBean() 
    {
        int breite = 450;
        int hoehe  = 30;

        wert        = 0.0;
        bezeichnung = "Zahl";        

        minwert = 0.0;
        maxwert = 100.0;

        textfeld  = new Textfeld(""+wert);
        aufkleber = new JLabel(bezeichnung); 
        schieber  = new Schieber(300);


        setLayout(new GridLayout(1,3));

        setBounds(10,60,breite,hoehe);
        add(aufkleber);
        add(schieber);
        add(textfeld);

        setBackground( Color.BLUE );
        aufkleber.setForeground( Color.WHITE );
    } 

    public class Textfeld extends TextField
    {
        Textfeld(String text)
        {
            super(text);
            enableEvents (AWTEvent.ACTION_EVENT_MASK);             
            enableEvents (AWTEvent.TEXT_EVENT_MASK);             
            enableEvents (AWTEvent.FOCUS_EVENT_MASK);             
        }

        protected void processEvent(AWTEvent e) 
        { 
             setWertIntern(getText());
        } 
    }

    public class Schieber extends JScrollBar
    {
        private int max_iwert;

        public void aktivieren()
        {
            addAdjustmentListener(new SchieberLauscher());
        }

        public void setWert(double wert, double minwert, double maxwert)
        {
            setValue( (int)((double)max_iwert*((wert-minwert)/(maxwert-minwert))) ); 
        }

        Schieber(int max_iwert)
        {
            super(JScrollBar.HORIZONTAL,0,10,0,max_iwert+10);        
            aktivieren();
            this.max_iwert = max_iwert;
        }

        public class SchieberLauscher implements AdjustmentListener 
        {        
            public void adjustmentValueChanged(AdjustmentEvent e) 
            {
                  setWertIntern(e.getValue(),max_iwert);
            }
        }
    }
}
